import 'dart:io';

import 'package:dial_videocall/src/utils/signaling.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';
import 'package:flutter_webrtc/webrtc.dart';

class VideoCallWidget extends StatefulWidget {
  VideoCallWidget({
    Key key, 
    @required this.server, 
    this.sessionName, 
    @required this.sessionFrom,
    this.userName, 
    this.secret, 
    this.iceServer
  }) : super(key: key);
  final String server;
  final String sessionFrom;
  final String sessionName;
  final String userName;
  final String secret;
  final String iceServer;

  @override
  _VideoCallWidgetState createState() => _VideoCallWidgetState();
}

class _VideoCallWidgetState extends State<VideoCallWidget> {

  final _localRenderer = new RTCVideoRenderer();
  final _remoteRenderer = new RTCVideoRenderer();
  bool _muted = false;
  bool _isCallIncoming = false;
  bool _isCallConnected = false;

  Signaling _signaling;
  String _callData;
  String _recordingId;

  @override
  void initState() {
    super.initState();
    initRenderers();
  }

  Future<void> initRenderers() async {
    await _localRenderer.initialize();
    await _remoteRenderer.initialize();
    _connect();
  }

  void _hangUp() {
    if (_signaling != null) {
      if(_recordingId!=null) _signaling.stopRecord(_recordingId); // Record handled on web app
      Navigator.of(context).pop();
    }
  }

  void _switchCamera() {
    _signaling.switchCamera();
  }

  void _muteMic() {
    setState(() {
      _muted = !_muted;
    });
    _signaling.muteMic(_muted);
  }
  
  @override
  Widget build(BuildContext context) {
    if(_isCallConnected){
      return _videoCallUI();
    }else if (_isCallIncoming){
      return _incomingCallUI();
    }else{
      return _callConnectingUI();
    }
  }

  Widget _incomingCallUI(){
    return Scaffold(
      body: Container(
        width: MediaQuery.of(context).size.width,
        color: Colors.black,
        child: Column(
          mainAxisAlignment: MainAxisAlignment.center,
          crossAxisAlignment: CrossAxisAlignment.center,
          children: [
            Text(
              "Telepon masuk dari ${widget.sessionFrom}",
              style: TextStyle(
                color: Colors.white,
                fontSize: 20
              ),
            ),
            SizedBox(height:50),
            Row(
              mainAxisAlignment: MainAxisAlignment.center,
              children: [
                ElevatedButton(
                  onPressed: _onAcceptCall,
                  child: Text("Accept", style: TextStyle(color: Colors.white),),
                  style: ButtonStyle(
                    backgroundColor: MaterialStateProperty.all<Color>(Colors.green),
                  ),
                ),
                SizedBox(width:30),
                ElevatedButton(
                  onPressed: _onRejectCall,
                  child: Text("Reject", style: TextStyle(color: Colors.white),),
                  style: ButtonStyle(
                    backgroundColor: MaterialStateProperty.all<Color>(Colors.red),
                  ),
                ),
              ],
            )
          ],
        ),
      ),
    );
  }

  Widget _callConnectingUI(){
    return Scaffold(
      body: Container(
        width: MediaQuery.of(context).size.width,
        color: Colors.black,
        child: Column(
          mainAxisAlignment: MainAxisAlignment.center,
          crossAxisAlignment: CrossAxisAlignment.center,
          children: [
            GestureDetector(
              child: Text(
                "Menghubungkan....",
                style: TextStyle(
                  color: Colors.white,
                  fontSize: 20
                ),
              ),
              onTap: (){
                setState(() {
                  _isCallIncoming = true;
                });
              },
            ),
            SizedBox(height:50),
            Platform.isAndroid? CircularProgressIndicator() : CupertinoActivityIndicator()
          ],
        ),
      ),
    );
  }

  Widget _videoCallUI(){
    return new Scaffold(
      body: OrientationBuilder(builder: (context, orientation) {
        return new Container(
          child: new Stack(children: <Widget>[
            new Positioned(
              left: 0.0,
              right: 0.0,
              top: 0.0,
              bottom: 0.0,
              child: new Container(
                margin: new EdgeInsets.fromLTRB(0.0, 0.0, 0.0, 0.0),
                width: MediaQuery.of(context).size.width,
                height: MediaQuery.of(context).size.height,
                child: new RTCVideoView(_remoteRenderer),
                decoration: new BoxDecoration(color: Colors.black54),
              )
            ),
            new Positioned(
              left: 20.0,
              top: 40.0,
              child: new Container(
                width: orientation == Orientation.portrait ? 90.0 : 120.0,
                height: orientation == Orientation.portrait ? 120.0 : 90.0,
                child: new RTCVideoView(_localRenderer),
                decoration: new BoxDecoration(color: Colors.black54),
              ),
            ),
          ]),
        );
      }),
      floatingActionButtonLocation: FloatingActionButtonLocation.centerFloat,
      floatingActionButton: new SizedBox(
        width: 200.0,
        child: new Row(
          mainAxisAlignment: MainAxisAlignment.spaceBetween,
          children: <Widget>[
            FloatingActionButton(
              child: const Icon(Icons.switch_camera),
              onPressed: _switchCamera,
              heroTag: "btn_switchCamera",
            ),
            FloatingActionButton(
              onPressed: _hangUp,
              heroTag: "btn_hangUp",
              tooltip: 'Hangup',
              child: new Icon(Icons.call_end),
              backgroundColor: Colors.pink,
            ),
            FloatingActionButton(
              child: _muted
              ? Icon(Icons.mic)
              : Icon(Icons.mic_off),
              onPressed: _muteMic,
              heroTag: "btn_muteMic",
              backgroundColor: _muted
              ? Colors.pink
              : Colors.blue,
            )
          ]
        )
      ),
    );
  }

  @override
  void dispose() {
    super.dispose();
    _signaling?.close();
    _localRenderer?.dispose();
    _remoteRenderer?.dispose();
    print('◤◢◤◢◤◢◤◢◤◢◤ dispose() ALL ◤◢◤◢◤◢◤◢◤◢◤');
  }

  void _onAcceptCall() async {
    var sessionData = widget.sessionName + "|" + _signaling.getUserID();
    _signaling.sendSignal(sessionId: sessionData, callData: _callData, type: "ACCEPTED_CALL");
    _signaling.muteMic(false);
    setState(() {
      _isCallIncoming = false;
      _isCallConnected = true;
    });
    _recordingId = await _signaling.startRecord(sessionId: widget.sessionName); // Record handled on web app
  }

  void _onRejectCall(){
    _signaling.sendSignal(sessionId: widget.sessionName, callData: _callData, type: "DECLINED_CALL");
    setState(() {
      _isCallIncoming = false;
      _isCallConnected = false;
    });
    _hangUp();
  }

  Future<void> _connect() async {
    
    if (_signaling == null) {

      //_Init Signaling
      _signaling = new Signaling(widget.server, widget.secret, widget.userName, widget.iceServer);

      //_Create session
      final String sessionId = await _signaling.createWebRtcSession(sessionId: widget.sessionName);
      print('◤◢◤◢◤◢◤◢◤◢◤ sessionId: $sessionId  ◤◢◤◢◤◢◤◢◤◢◤ ');

      //_Create_token
      final String token = await _signaling.createWebRtcToken(sessionId: sessionId);
      print('◤◢◤◢◤◢◤◢◤◢◤ token: $token  ◤◢◤◢◤◢◤◢◤◢◤ ');
    
      //_Connect_socket //ADDED AWAIT
      await _signaling.connect();

      _signaling.onStateChange = (SignalingState state) {
        print('_signaling.onStateChange: $state');
        switch (state) {
          case SignalingState.CallStateNew:
            break;
          case SignalingState.CallStateBye:
            break;
          default:
            break;
        }
      };

      _signaling.onLocalStream = ((stream) {
        print('onLocalStream: ${stream.id}');
        setState(() {
          _localRenderer.srcObject = stream;
        });
      });

      _signaling.onAddRemoteStream = ((stream) {
        print('onAddRemoteStream: ${stream.id}');
        setState(() {
          _remoteRenderer.srcObject = stream;
        });
      });

      _signaling.onRemoveRemoteStream = ((stream) {
        print('onRemoveRemoteStream');
        _remoteRenderer.srcObject = null;
      });

      // Send signal Ready to call
      _signaling.checkSession(sessionId: widget.sessionFrom)
        .then((sessionAvailable) {
          if(sessionAvailable){
            _signaling.sendSignal(sessionId: "ready", callData: widget.sessionFrom, type: "READY_TO_CALL");
          }else{
            _hangUp();
          }
        });

      // Handle incoming signal / message
      _signaling.onMessageReceive = ((stream){
        String type = stream['type'];
        String data = stream['data'];
        if(type == "CALL_REQUEST"){
          _callData = data;
          _signaling.sendSignal(sessionId: sessionId, callData: _callData, type: "CALL_REQUEST");
          setState(() {
            _isCallIncoming = true;
            _isCallConnected = false;
          });
        }else if (type == "END_CALL"){
          setState(() {
            _isCallIncoming = false;
            _isCallConnected = false;
          });
          _hangUp();
        }
      });
    }
  }
}
